using Json2CSharpCodeGenerator.Lib;
using Json2CSharpCodeGenerator.Lib.CodeWriters;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Json2CSharpCodeGenerator.Tests;

[TestClass]
public class Test_17_RECORD_TYPES
{
    [TestMethod]
    public void Run() => DoTest(null);

    [TestMethod]
    public void RunNewtonsoft() => DoTest(false, "_NEWTONSOFT");

    [TestMethod]
    public void RunSystemTextJson() => DoTest(true, "_SYSTEMTEXTJSON");

    private static void DoTest(bool? useSystemTextJson, string outputFileSuffix = "")
    {
        string inputPath = Path.Combine(AppContext.BaseDirectory, "Test_17_RECORD_TYPES_INPUT.txt");
        string outputPath = Path.Combine(AppContext.BaseDirectory, $"Test_17_RECORD_TYPES_OUTPUT{outputFileSuffix}.txt");
        string input = File.ReadAllText(inputPath);
        string output = File.ReadAllText(outputPath);

        JsonClassGenerator jsonClassGenerator = new()
        {
            CodeWriter = new CSharpCodeWriter(),
            OutputType = OutputTypes.ImmutableRecord
        };

        if (useSystemTextJson.HasValue)
        {
            jsonClassGenerator.AttributeUsage = JsonPropertyAttributeUsage.Always;

            if (useSystemTextJson.Value)
            {
                jsonClassGenerator.AttributeLibrary = JsonLibrary.SystemTextJson;
            }
            else
            {
                jsonClassGenerator.AttributeLibrary = JsonLibrary.NewtonsoftJson;
            }
        }
        else
        {
            jsonClassGenerator.AttributeUsage = JsonPropertyAttributeUsage.OnlyWhenNecessary;
        }

        string actual = jsonClassGenerator.GenerateClasses(input, out _).ToString();
        // Console.WriteLine("Actual:\n" + actual);

        string outputFixed = output.NormalizeOutput();
        string actualFixed = actual.NormalizeOutput();

        Assert.AreEqual(expected: outputFixed, actual: actualFixed);
    }
}